@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2001, 2002, 2003, 2004  Thorsten Kukuk
+/* Copyright (c) 2000-2005, 2011, 2014, 2015, 2016  Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -12,20 +12,16 @@
 
    You should have received a copy of the GNU General Public
    License along with the YP Server; see the file COPYING. If
-   not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-   Cambridge, MA 02139, USA. */
+   not, write to the Free Software Foundation, Inc., 51 Franklin Street,
+   Suite 500, Boston, MA 02110-1335, USA. */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#define _GNU_SOURCE
-
 #include <unistd.h>
 #include <syslog.h>
-#if defined(HAVE_GETOPT_H)
 #include <getopt.h>
-#endif /* HAVE_GETOPT_H */
 #include <netdb.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -35,9 +31,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <ctype.h>
-#ifdef HAVE_ALLOCA_H
 #include <alloca.h>
-#endif /* HAVE_ALLOCA_H */
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/param.h>
@@ -46,10 +40,13 @@
 #include "ypxfr.h"
 #include "ypxfrd.h"
 #include <rpcsvc/ypclnt.h>
-#include "compat.h"
 
+#if defined(HAVE_COMPAT_LIBGDBM)
 #if defined(HAVE_LIBGDBM)
 #include <gdbm.h>
+#elif defined(HAVE_LIBQDBM)
+#include <hovel.h>
+#endif
 
 #define ypdb_store gdbm_store
 #define YPDB_REPLACE GDBM_REPLACE
@@ -64,10 +61,45 @@ static GDBM_FILE dbm;
 #define ypdb_close dbm_close
 #define ypdb_fetch dbm_fetch
 static DBM *dbm;
-#endif
+#elif defined(HAVE_LIBTC)
+#include <tcbdb.h>
+
+#define YPDB_REPLACE 1
+
+static inline int
+ypdb_close (TCBDB *dbp)
+{
+  tcbdbclose (dbp);
+  tcbdbdel (dbp);
+  dbp = NULL;
+  return 0;
+}
+
+static int
+ypdb_store (TCBDB *dbm, datum key, datum data, int mode)
+{
+  if (mode != YPDB_REPLACE)
+    return 1;
+
+  return !tcbdbput(dbm, key.dptr, key.dsize, data.dptr, data.dsize);
+}
+
+static datum
+ypdb_fetch (TCBDB *bdb, datum key)
+{
+  datum res;
+
+  if (!(res.dptr = tcbdbget(bdb, key.dptr, key.dsize, &res.dsize)))
+    {
+      res.dptr = NULL;
+      res.dsize = 0;
+    }
+
+  return res;
+}
+
+static TCBDB *dbm;
 
-#ifndef YPMAPDIR
-#define YPMAPDIR "/var/yp"
 #endif
 
 static char *path_ypdb = YPMAPDIR;
@@ -198,6 +230,35 @@ xdr_ypxfr_xfr (XDR *xdrs, xfr *objp)
     }
 }
 
+#ifdef HAVE_RPCB_GETADDR
+static unsigned short
+__taddr2port (const struct netconfig *nconf, const struct netbuf *nbuf)
+{
+  unsigned short port = 0;
+  struct __rpc_sockinfo si;
+  struct sockaddr_in *sin;
+  struct sockaddr_in6 *sin6;
+  if (!__rpc_nconf2sockinfo(nconf, &si))
+    return 0;
+
+  switch (si.si_af)
+    {
+    case AF_INET:
+      sin = nbuf->buf;
+      port = sin->sin_port;
+      break;
+    case AF_INET6:
+      sin6 = nbuf->buf;
+      port = sin6->sin6_port;
+      break;
+    default:
+      break;
+    }
+
+  return htons (port);
+}
+#endif
+
 static int
 ypxfrd_transfer (char *host, char *map, char *domain, char *tmpname)
 {
@@ -205,17 +266,62 @@ ypxfrd_transfer (char *host, char *map, char *domain, char *tmpname)
   struct ypxfr_mapname req;
   struct xfr resp;
   struct timeval timeout = {25, 0};
+  int port = 0;
+#if defined(HAVE_RPCB_GETADDR)
+  struct netconfig *nconf;
+  struct netbuf svcaddr;
+  char addrbuf[INET6_ADDRSTRLEN];
+  void *handle;
+  int found;
+#endif
 
   if (debug_flag)
     fprintf (stderr, "Trying ypxfrd ...");
 
-  if (!getrpcport (host, YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS,
-                   IPPROTO_TCP))
+#ifdef HAVE_RPCB_GETADDR
+  svcaddr.len = 0;
+  svcaddr.maxlen = sizeof (addrbuf);
+  svcaddr.buf = addrbuf;
+  found = 0;
+
+  handle = setnetconfig();
+  while ((nconf = getnetconfig(handle)) != NULL)
     {
-      if (debug_flag)
-        log_msg (" not running\n");
+      if (!strcmp(nconf->nc_proto, "udp"))
+	{
+	  if (rpcb_getaddr(YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS,
+			   nconf, &svcaddr, host))
+	    {
+	      port = __taddr2port (nconf, &svcaddr);
+	      endnetconfig (handle);
+	      found=1;
+	      break;
+	    }
+
+	  if (rpc_createerr.cf_stat != RPC_UNKNOWNHOST)
+	    {
+	      clnt_pcreateerror (host);
+	      log_msg ("rpcb_getaddr (%s) failed!", host);
+	      return 1;
+	    }
+	}
+    }
+
+  if (!found)
+    {
+      log_msg ("Cannot find suitable transport for protocol 'udp'");
       return 1;
     }
+#else
+	port = getrpcport (host, YPXFRD_FREEBSD_PROG, YPXFRD_FREEBSD_VERS,
+                   IPPROTO_TCP))
+#endif
+    if (port == 0)
+      {
+	if (debug_flag)
+	  log_msg (" not running");
+	return 1;
+      }
 
   req.xfrmap = map;
   req.xfrdomain = domain;
@@ -243,6 +349,16 @@ ypxfrd_transfer (char *host, char *map, char *domain, char *tmpname)
   req.xfr_db_type = XFR_DB_BSD_NDBM;
   req.xfr_byte_order = XFR_ENDIAN_ANY;
 #endif
+#elif defined (HAVE_LIBQDBM)
+  req.xfr_db_type = XFR_DB_QDBM;
+#if defined(WORDS_BIGENDIAN)
+  req.xfr_byte_order = XFR_ENDIAN_BIG;
+#else
+  req.xfr_byte_order = XFR_ENDIAN_LITTLE;
+#endif
+#elif defined (HAVE_LIBTC)
+  req.xfr_db_type = XFR_DB_TC;
+  req.xfr_byte_order = XFR_ENDIAN_ANY;
 #endif
   memset (&resp, 0, sizeof (resp));
 
@@ -282,16 +398,9 @@ error:
   return 1;
 }
 
-/* NetBSD has a different prototype in struct ypall_callback */
-#if defined(__NetBSD__)
-static int
-ypxfr_foreach (u_long status, char *key, int keylen,
-               char *val, int vallen, void *data UNUSED)
-#else
 static int
 ypxfr_foreach (int status, char *key, int keylen,
                char *val, int vallen, char *data UNUSED)
-#endif
 {
   datum outKey, outData;
 
@@ -347,19 +456,17 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
   char dbName_orig[MAXPATHLEN + 1];
   char dbName_temp[MAXPATHLEN + 1];
   char *master_host = NULL;
+  char *server = NULL;
   struct ypreq_key req_key;
   struct ypresp_val resp_val;
   struct ypresp_all resp_all;
   datum outKey, outData;
-  struct timeval TIMEOUT = {25, 0};
   CLIENT *clnt_udp;
-  struct sockaddr_in sockaddr, sockaddr_udp;
   struct ypresp_order resp_order;
   struct ypresp_master resp_master;
   struct ypreq_nokey req_nokey;
   time_t masterOrderNum;
-  struct hostent *h;
-  int sock, result;
+  int result;
 
   /* Name of the map file */
   if (strlen (path_ypdb) + strlen (target_domain) + strlen (map) + 3 < MAXPATHLEN)
@@ -381,14 +488,8 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
 
   /* Build a connection to the host with the master map, not to
      the nearest ypserv, because this map could be out of date. */
-  memset (&sockaddr, '\0', sizeof (sockaddr));
-  sockaddr.sin_family = AF_INET;
   if (source_host)
-    {
-      h = gethostbyname (source_host);
-      if (!h)
-	return YPXFR_RSRC;
-    }
+    server = source_host;
   else
     {
       /* Get the Master hostname for the map, if no explicit
@@ -397,17 +498,10 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
 
       if (yp_master (source_domain, map, &master_name))
         return YPXFR_MADDR;
-      h = gethostbyname (master_name);
+      server = strdupa (master_name);
       free (master_name);
-      if (!h)
-	return YPXFR_RSRC;
     }
-  memcpy (&sockaddr.sin_addr, h->h_addr, sizeof sockaddr.sin_addr);
-  memcpy (&sockaddr_udp, &sockaddr, sizeof (sockaddr));
-
-  /* Create a udp socket to the server */
-  sock = RPC_ANYSOCK;
-  clnt_udp = clntudp_create (&sockaddr_udp, YPPROG, YPVERS, TIMEOUT, &sock);
+  clnt_udp = clnt_create (server, YPPROG, YPVERS, "udp");
   if (clnt_udp == NULL)
     {
       log_msg (clnt_spcreateerror ("YPXFR"));
@@ -419,16 +513,17 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
      server name for the map on the master host. */
   req_nokey.domain = source_domain;
   req_nokey.map = map;
+  memset (&resp_master, 0, sizeof (resp_master));
   if (ypproc_master_2 (&req_nokey, &resp_master, clnt_udp) != RPC_SUCCESS)
     {
       log_msg (clnt_sperror (clnt_udp, "ypproc_master_2"));
       clnt_destroy (clnt_udp);
       return YPXFR_YPERR;
     }
-  else if (resp_master.stat != YP_TRUE)
+  else if (resp_master.status != YP_TRUE)
     {
       clnt_destroy (clnt_udp);
-      switch (resp_master.stat)
+      switch (resp_master.status)
 	{
 	case YP_NOMAP:
 	  return YPXFR_NOMAP;
@@ -441,14 +536,14 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
 	case YP_BADARGS:
 	  return YPXFR_BADARGS;
 	default:
-	  log_msg ("ERROR: not expected value: %s", resp_master.stat);
+	  log_msg ("ERROR: not expected value: %s", resp_master.status);
 	  return YPXFR_XFRERR;
 	}
     }
   else
     {
-      master_host = alloca (strlen (resp_master.peer) + 1);
-      strcpy (master_host, resp_master.peer);
+      master_host = alloca (strlen (resp_master.master) + 1);
+      strcpy (master_host, resp_master.master);
       xdr_free ((xdrproc_t) xdr_ypresp_master, (caddr_t) &resp_master);
     }
 
@@ -457,16 +552,18 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
      for the map on the master host. */
   req_nokey.domain = source_domain;
   req_nokey.map = map;
+  memset (&resp_order, 0, sizeof (resp_order));
   if (ypproc_order_2 (&req_nokey, &resp_order, clnt_udp) != RPC_SUCCESS)
     {
       log_msg (clnt_sperror (clnt_udp, "ypproc_order_2"));
-      masterOrderNum = time (NULL); /* We set it to the current time.
-                                       So a new map will be always newer. */
+      clnt_destroy (clnt_udp);
+	  return YPXFR_YPERR; /* return error when not possible to
+                            get masterOrder */
     }
-  else if (resp_order.stat != YP_TRUE)
+  else if (resp_order.status != YP_TRUE)
     {
       clnt_destroy (clnt_udp);
-      switch (resp_order.stat)
+      switch (resp_order.status)
 	{
 	case YP_NOMAP:
 	  return YPXFR_NOMAP;
@@ -479,7 +576,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
 	case YP_BADARGS:
 	  return YPXFR_BADARGS;
 	default:
-	  log_msg ("ERROR: not expected value: %s", resp_order.stat);
+	  log_msg ("ERROR: not expected value: %s", resp_order.status);
 	  return YPXFR_XFRERR;
 	}
     }
@@ -495,10 +592,17 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
       time_t localOrderNum = 0;
       datum inKey, inVal;
 
-#if defined(HAVE_LIBGDBM)
+#if defined(HAVE_COMPAT_LIBGDBM)
       dbm = gdbm_open (dbName_orig, 0, GDBM_READER, 0600, NULL);
 #elif defined(HAVE_NDBM)
-      dbm = dbm_open (dbName_orig, O_CREAT|O_RDWR, 0600);
+      dbm = dbm_open (dbName_orig, O_RDONLY, 0600);
+#elif defined(HAVE_LIBTC)
+      dbm = tcbdbnew ();
+      if (!tcbdbopen (dbm, dbName_orig, BDBOREADER | BDBONOLCK))
+        {
+          tcbdbdel (dbm);
+          dbm = NULL;
+        }
 #endif
       if (dbm == NULL)
         {
@@ -557,10 +661,17 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
       char orderNum[255];
       CLIENT *clnt_tcp;
 
-#if defined(HAVE_LIBGDBM)
+#if defined(HAVE_COMPAT_LIBGDBM)
       dbm = gdbm_open (dbName_temp, 0, GDBM_NEWDB, 0600, NULL);
 #elif defined(HAVE_NDBM)
       dbm = dbm_open (dbName_temp, O_CREAT|O_RDWR, 0600);
+#elif defined(HAVE_LIBTC)
+      dbm = tcbdbnew ();
+      if (!tcbdbopen (dbm, dbName_temp, BDBOWRITER | BDBOCREAT | BDBOTRUNC))
+        {
+          tcbdbdel (dbm);
+          dbm = NULL;
+        }
 #endif
       if (dbm == NULL)
         {
@@ -598,8 +709,9 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
 	 a NIS slave server and request the map from us. */
       req_key.domain = source_domain;
       req_key.map = map;
-      req_key.key.keydat_val = "YP_INTERDOMAIN";
-      req_key.key.keydat_len = strlen ("YP_INTERDOMAIN");
+      req_key.keydat.keydat_val = "YP_INTERDOMAIN";
+      req_key.keydat.keydat_len = strlen ("YP_INTERDOMAIN");
+      memset (&resp_val, 0, sizeof (resp_val));
       if (ypproc_match_2 (&req_key, &resp_val, clnt_udp) != RPC_SUCCESS)
         {
           clnt_perror (clnt_udp, "ypproc_match(YP_INTERDOMAIN)");
@@ -610,7 +722,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
         }
       else
         {
-          if (resp_val.stat == YP_TRUE)
+          if (resp_val.status == YP_TRUE)
             {
               outKey.dptr = "YP_INTERDOMAIN";
               outKey.dsize = strlen (outKey.dptr);
@@ -631,8 +743,9 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
       /* Get the YP_SECURE field. */
       req_key.domain = source_domain;
       req_key.map = map;
-      req_key.key.keydat_val = "YP_SECURE";
-      req_key.key.keydat_len = strlen ("YP_SECURE");
+      req_key.keydat.keydat_val = "YP_SECURE";
+      req_key.keydat.keydat_len = strlen ("YP_SECURE");
+      memset (&resp_val, 0, sizeof (resp_val));
       if (ypproc_match_2 (&req_key, &resp_val, clnt_udp) != RPC_SUCCESS)
         {
           clnt_perror (clnt_udp, "yproc_match");
@@ -643,7 +756,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
         }
       else
         {
-          if (resp_val.stat == YP_TRUE)
+          if (resp_val.status == YP_TRUE)
             {
               outKey.dptr = "YP_SECURE";
               outKey.dsize = strlen (outKey.dptr);
@@ -664,8 +777,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
       /* We don't need clnt_udp any longer, give it free */
       clnt_destroy (clnt_udp);
       /* Create a tcp socket to the server */
-      sock = RPC_ANYSOCK;
-      clnt_tcp = clnttcp_create (&sockaddr, YPPROG, YPVERS, &sock, 0, 0);
+      clnt_tcp = clnt_create (server, YPPROG, YPVERS, "tcp");
       if (clnt_tcp == NULL)
 	{
 	  clnt_pcreateerror ("YPXFR");
@@ -678,6 +790,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
         req_nokey.domain = source_domain;
         req_nokey.map = map;
         xdr_ypall_callback = &callback;
+	memset (&resp_all, 0, sizeof (resp_all));
         if (ypproc_all_2 (&req_nokey, &resp_all, clnt_tcp) != RPC_SUCCESS)
           {
             clnt_perror (clnt_tcp, "ypall");
@@ -687,14 +800,14 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
           }
         else
           {
-            switch (resp_all.ypresp_all_u.val.stat)
+            switch (resp_all.ypresp_all_u.val.status)
               {
               case YP_TRUE:
               case YP_NOMORE:
                 result = 0;
                 break;
               default:
-                result = ypprot_err (resp_all.ypresp_all_u.val.stat);
+                result = ypprot_err (resp_all.ypresp_all_u.val.status);
               }
             clnt_freeres (clnt_tcp, (xdrproc_t) ypxfr_xdr_ypresp_all,
 			  (caddr_t) &resp_all);
@@ -706,7 +819,12 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
     }
 
   if (result == 0)
-    rename (dbName_temp, dbName_orig);
+    {
+#if defined(HAVE_LIBTC)
+      chmod(dbName_temp, S_IRUSR|S_IWUSR);
+#endif
+      rename (dbName_temp, dbName_orig);
+    }
   else
     unlink(dbName_temp);
 
@@ -732,7 +850,7 @@ int
 main (int argc, char **argv)
 {
   char *source_host = NULL, *target_domain = NULL, *source_domain = NULL;
-  struct in_addr remote_addr;
+  const char *remote_addr = NULL;
   unsigned int transid = 0;
   unsigned short int remote_port = 0;
   unsigned long program_number = 0;
@@ -747,8 +865,6 @@ main (int argc, char **argv)
   else
     debug_flag = 1;
 
-  memset (&remote_addr, 0, sizeof (remote_addr));
-
   while (1)
     {
       int c;
@@ -806,7 +922,7 @@ main (int argc, char **argv)
 	    }
 	  transid = atoi (optarg);
 	  program_number = atoi (argv[optind++]);
-	  remote_addr.s_addr = inet_addr (argv[optind++]);
+	  remote_addr = argv[optind++];
 	  remote_port = atoi (argv[optind++]);
 	  break;
 	case 'u':
@@ -828,7 +944,11 @@ main (int argc, char **argv)
 
   if (target_domain == NULL)
     {
-      yp_get_default_domain (&target_domain);
+      if (yp_get_default_domain (&target_domain) != 0)
+	{
+	  log_msg ("Cannot get default domain");
+	  exit (1);
+	}
 
       if (target_domain == NULL)
 	{
@@ -857,35 +977,33 @@ main (int argc, char **argv)
       /* Now send the status to the yppush program, so it can display a
 	 message for the sysop and do not timeout. */
       if (transid)
-        {
-          struct sockaddr_in addr;
+	{
+	  struct timeval tv = {10, 0};
           CLIENT *clnt;
-          int s;
           ypresp_xfr resp;
-          static struct timeval tv = {10, 0};
 
-          memset (&addr, '\0', sizeof addr);
-          addr.sin_addr = remote_addr;
-          addr.sin_port = htons (remote_port);
-          addr.sin_family = AF_INET;
-          s = RPC_ANYSOCK;
-
-          clnt = clntudp_create (&addr, program_number, 1, tv, &s);
+          clnt = clnt_create (remote_addr, program_number, YPPUSHVERS, "datagram_n");
           if (!clnt)
             {
-              clnt_pcreateerror ("ypxfr_callback");
+              clnt_pcreateerror ("ypxfr_callback create");
+	      if (debug_flag)
+		log_msg ("Remote host: %s, %x", remote_addr, program_number);
               continue;
             }
 	  resp.transid = transid;
           resp.xfrstat = res;
 
-          if (clnt_call (clnt, 1, (xdrproc_t) xdr_ypresp_xfr, (caddr_t) &resp,
+          if (clnt_call (clnt, YPPUSHPROC_XFRRESP,
+			 (xdrproc_t) xdr_ypresp_xfr, (caddr_t) &resp,
                          (xdrproc_t) xdr_void, (caddr_t)&res, tv)
 	      != RPC_SUCCESS)
-            clnt_perror (clnt, "ypxfr_callback");
+	    {
+	      clnt_perror (clnt, "ypxfr_callback call");
+	      if (debug_flag)
+		log_msg ("Remote host: %s, %x", remote_addr, program_number);
+	    }
 
           clnt_destroy (clnt);
-
 	}
     }
 
