@@ -12,15 +12,13 @@
 
    You should have received a copy of the GNU General Public
    License along with the YP Server; see the file COPYING. If
-   not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-   Cambridge, MA 02139, USA. */
+   not, write to the Free Software Foundation, Inc., 51 Franklin Street,
+   Suite 500, Boston, MA 02110-1335, USA. */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#define _GNU_SOURCE
-
 #include <time.h>
 #include <string.h>
 #include <memory.h>
@@ -33,6 +31,7 @@ static struct timeval TIMEOUT = { 25, 0 };
 enum clnt_stat
 ypproc_order_2 (ypreq_nokey *argp, ypresp_order *clnt_res, CLIENT *clnt)
 {
+  memset(clnt_res, 0, sizeof(ypresp_order));
   return (clnt_call(clnt, YPPROC_ORDER,
 		    (xdrproc_t) xdr_ypreq_nokey, (caddr_t) argp,
 		    (xdrproc_t) xdr_ypresp_order, (caddr_t) clnt_res,
@@ -42,6 +41,7 @@ ypproc_order_2 (ypreq_nokey *argp, ypresp_order *clnt_res, CLIENT *clnt)
 enum clnt_stat
 ypproc_master_2 (ypreq_nokey *argp, ypresp_master *clnt_res, CLIENT *clnt)
 {
+  memset(clnt_res, 0, sizeof(ypresp_master));
   return (clnt_call(clnt, YPPROC_MASTER,
 		    (xdrproc_t) xdr_ypreq_nokey, (caddr_t) argp,
 		    (xdrproc_t) xdr_ypresp_master, (caddr_t) clnt_res,
